/******************************************************************************
 *
 * This file is part of the DCES project.
 * See http://www.ohwr.org/projects/dces-dtrhf-ser1ch-v1
 *
 * Copyright (C) 2016 CERN
 * All rights not expressly granted are reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author DCES Dev team, dces-dev@cern.ch
 *****************************************************************************/


#include <lib_fan.h>

// GLOBALS
unsigned int RPM_Measure_Timeout = RPM_TIMEOUT;	// Used to set a measure timeout
unsigned int i_Cnt;				// Used to count interrupts (2 per revolution)
unsigned long s_Time, e_Time, d_Time;		// Used to measure time per revolution: start, end, delta
bool b_Cnt = false;				// Used to know whether the measure is finished

// INIT FAN PWM AND TACHOMETER
void initFan(void){
  DDRB |= (1 << DDB6);       // Set PB6 as an output (PWM output, pin 12) 
  DDRD &= ~(1 << DDD2);      // Set PD2 as an input (Tachometer input, pin 19)

  TCCR1A |= (1 << COM1B1);   // Configures the Timer/Counter 1 for fast PWM
  TCCR1A |= (1 << WGM11);
  TCCR1A |= (1 << WGM10);

  TCCR1B |= (1 << WGM13);
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS10);

  OCR1A = 639;		     // Set the top value of PWM register to set frequency at 25kHz
  OCR1B = 639/100*FAN_SPEED; // Set the default duty-cycle depending on #define FAN_SPEED (in %)
  
  noInterrupts();            // Configures interruption for tachometer
  EICRA |= (1 << ISC21);     // Here, on falling edge
  interrupts();
}


// GET THE RPM OF THE FAN
unsigned int getRPM(void){
  unsigned int RPM_Measure = 0;	    // Will contain the value of instant RPMs
  unsigned int i_Bcl = 0;	    // Used to measure the timeout
  noInterrupts();                   // Enable interruptions for tachometer
  EIMSK |= (1 << INT2);		    
  interrupts();

  for(i_Bcl=0; i_Bcl<=RPM_Measure_Timeout; i_Bcl++){     // Watch dog for RPM measure Timeout
    delay(1);
    if(b_Cnt==true){                      // If the measure is finished
      d_Time = (e_Time-s_Time);           // Then, compute the results to get duration of one revolution
      RPM_Measure = (1000000/d_Time)*60;  // Compute the duration to get instant RPMs
      b_Cnt = false;
      break;
    }
  }

  if(i_Bcl>=RPM_Measure_Timeout){         // If there is a real timeout on the RPM measure (fan blocked)  
    noInterrupts();                       // Disable interruptions
    EIMSK &= ~(1 << INT2);
    interrupts();
    i_Cnt = 0;                            // Reset counting variable
    RPM_Measure = 0;                      // Set RPMs to 0 (Timeout...)
  }
  return RPM_Measure;
}

// SET PWM DUTY CYCLE IN %
void setRPM(unsigned int pct){		  
  unsigned int duty_cycle=0;		  // Temp variable
  if(pct>=0 && pct<=100){		  // If the value is between 0 and 100
    duty_cycle = pct*639/100;		  // Convert the percentage to the corresponding value on 639
  }
  else{
    duty_cycle =(639*FAN_SPEED)/100;      // If the value isn't in the interval, set the default value
  }
  OCR1B = duty_cycle;			  // Affect the value on 639 to the compare register
}

// INTERRUPTION SUBROUTINE (used to count interrupts generated by the fan -> 2 per revolution)
ISR(INT2_vect, ISR_BLOCK){
  i_Cnt++;
  if(i_Cnt==2 && b_Cnt==false){		  // If we are at the begginning of the revolution
    s_Time = micros();			  // Get the time
  }
  if(i_Cnt==4 && b_Cnt==false){		  // If we are at the end of the revolution
    e_Time = micros();			  // Get the time
    noInterrupts();			  
    EIMSK &= ~(1 << INT2);		  // Stop interruptions
    interrupts();
    b_Cnt = true;			  // Set b_Cnt to true to tell the measure is finished
    i_Cnt = 0;				  // Reset the counting variable
  }
}

// SET A DIFFERENT TIMEOUT FOR RPM MEASURE (in ms)
void setRPMTimeout(unsigned int timeout){
  if(timeout>=1 && timeout<=1000){
    RPM_Measure_Timeout = timeout;
  }
}

